import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
/**
 *
 *  Copyright (c) "Neo4j"
 *  Neo4j Sweden AB [http://neo4j.com]
 *
 *  This file is part of Neo4j.
 *
 *  Neo4j is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
import { forwardRef, useMemo, } from 'react';
import { classNames } from '../_common/defaultImports';
import { customLabelAndNoAriaLabelWarningMessage } from '../_common/messages';
import { needleWarningMessage } from '../_common/utils';
import { ExclamationCircleIconSolid, InformationCircleIconOutline, } from '../icons';
import { Skeleton } from '../skeleton';
import { Tooltip } from '../tooltip';
import { Typography } from '../typography';
/**
 *
 *
 * TextInput Component
 *
 *
 */
export const TextInput = forwardRef(function TextInput(props, ref) {
    const { label, isFluid, errorText, helpText, leftElement, rightElement, isOptional = false, informationIconText, size = 'medium', placeholder, value, tooltipProps, htmlAttributes, isDisabled, isReadOnly, isRequired, onChange, className, style, isSkeletonLoading = false, skeletonProps, } = props;
    const containerWrappingClasses = classNames(`ndl-text-input`, className, {
        'ndl-disabled': isDisabled,
        'ndl-read-only': isReadOnly,
        'ndl-small': size === 'small',
        'ndl-medium': size === 'medium',
        'ndl-large': size === 'large',
        'ndl-has-error': errorText,
        'ndl-has-left-icon': leftElement,
        'ndl-has-right-icon': rightElement || errorText,
        'ndl-has-icon': leftElement || rightElement || errorText,
    });
    const hasEmptyLabelValue = !label || label === '';
    const labelWrappingClasses = classNames('ndl-form-item-label', {
        'ndl-fluid': isFluid,
        'ndl-form-item-no-label': hasEmptyLabelValue,
    });
    const combinedHtmlInputAttributes = Object.assign(Object.assign({}, htmlAttributes), { className: classNames('ndl-input', htmlAttributes === null || htmlAttributes === void 0 ? void 0 : htmlAttributes.className) });
    const ariaLabel = combinedHtmlInputAttributes['aria-label'];
    const isCustomLabel = label && typeof label !== 'string';
    const hasCustomLabelAndNoAriaLabel = isCustomLabel && !ariaLabel;
    useMemo(() => {
        if (!label && !ariaLabel) {
            needleWarningMessage('A TextInput without a label does not have an aria label, be sure to include an aria label for screen readers. Link: https://dequeuniversity.com/rules/axe/4.2/label?application=axeAPI');
        }
        if (hasCustomLabelAndNoAriaLabel) {
            needleWarningMessage(customLabelAndNoAriaLabelWarningMessage);
        }
    }, [label, ariaLabel, hasCustomLabelAndNoAriaLabel]);
    const informationIconClasses = classNames({
        'ndl-information-icon-small': size === 'small' || size === 'medium',
        'ndl-information-icon-large': size === 'large',
    });
    return (_jsxs("div", { className: containerWrappingClasses, style: style, children: [_jsxs("label", { className: labelWrappingClasses, children: [!hasEmptyLabelValue && (_jsx(Skeleton, Object.assign({ onBackground: "weak", variant: "rectangular" }, skeletonProps, { isLoading: isSkeletonLoading, children: _jsxs("div", { className: "ndl-label-text-wrapper", children: [_jsx(Typography, { variant: size === 'large' ? 'body-large' : 'body-medium', className: "ndl-label-text", children: label }), informationIconText && (_jsxs(Tooltip, Object.assign({}, tooltipProps === null || tooltipProps === void 0 ? void 0 : tooltipProps.tooltipRoot, { type: "simple", children: [_jsx(Tooltip.Trigger, Object.assign({}, tooltipProps === null || tooltipProps === void 0 ? void 0 : tooltipProps.tooltipTrigger, { className: informationIconClasses, children: _jsx(InformationCircleIconOutline, {}) })), _jsx(Tooltip.Content, Object.assign({}, tooltipProps === null || tooltipProps === void 0 ? void 0 : tooltipProps.tooltipContent, { children: informationIconText }))] }))), isOptional && (_jsx(Typography, { variant: size === 'large' ? 'body-large' : 'body-medium', className: "ndl-form-item-optional", children: "Optional" }))] }) }))), _jsx(Skeleton, Object.assign({ onBackground: "weak", variant: "rectangular" }, skeletonProps, { isLoading: isSkeletonLoading, children: _jsxs("div", { className: "ndl-input-wrapper", children: [leftElement && (_jsx("div", { className: "ndl-element-left ndl-element", children: leftElement })), _jsx("input", Object.assign({ readOnly: isReadOnly, disabled: isDisabled, required: isRequired, value: value, placeholder: placeholder, type: "text", onChange: onChange }, combinedHtmlInputAttributes, { ref: ref })), rightElement && (_jsx("div", { className: "ndl-element-right ndl-element", children: rightElement }))] }) }))] }), helpText && !errorText && (_jsx(Skeleton, { onBackground: "weak", variant: "rectangular", isLoading: isSkeletonLoading, children: _jsx(Typography, { variant: size === 'large' ? 'body-medium' : 'body-small', className: "ndl-form-message", children: helpText }) })), errorText && (
            // TODO v4: We might want to have a min width for the container for the messages to help skeleton loading.
            // Currently the message fills 100% of the width while the rest of the text input has a set width.
            _jsx(Skeleton, Object.assign({ onBackground: "weak", variant: "rectangular", width: "fit-content" }, skeletonProps, { isLoading: isSkeletonLoading, children: _jsxs("div", { className: "ndl-form-message", children: [_jsx("div", { className: "ndl-error-icon", children: _jsx(ExclamationCircleIconSolid, {}) }), _jsx(Typography, { className: "ndl-error-text", variant: size === 'large' ? 'body-medium' : 'body-small', children: errorText })] }) })))] }));
});
//# sourceMappingURL=TextInput.js.map